#![allow(non_snake_case)]
extern crate reqwest;
extern crate rocket;

mod discord;
mod session;

use discord::discord::Client;
use dotenv::dotenv;
use rocket::fs::NamedFile;
use rocket::response::Redirect;
use rocket::serde::json::Json;
use rocket::tokio::sync::Mutex;
use rocket::State;
use rocket::{get, launch, routes};

use self::session::session::{Session, SessionManager};

type SharedSessionManager = Mutex<SessionManager>;

#[get("/callback?<code>")]
async fn index(
    shared_session_manager: &State<SharedSessionManager>,
    code: String,
    client: &State<Client>,
) -> Json<Session> {
    let token_exchange = client.exchange_code(&code).await.unwrap();
    let user = client.get_user(&token_exchange).await.unwrap();
    let mut session_manager = shared_session_manager.lock().await;
    let session = session_manager
        .generate_session(user, &token_exchange.access_token)
        .await;
    Json(session)
}

#[get("/favicon.ico")]
async fn get_favicon() -> Option<NamedFile> {
    NamedFile::open("./favicon.ico").await.ok()
}

#[get("/session?<token>")]
async fn return_sess(
    shared_session_manager: &State<SharedSessionManager>,
    token: String,
) -> Json<Session> {
    Json(
        shared_session_manager
            .lock()
            .await
            .get_session(&token)
            .await
            .unwrap(),
    )
}

#[get("/")]
async fn redirect_auth(client: &State<Client>) -> Redirect {
    Redirect::to(client.generate_authorize_url("identify", "code"))
}

#[launch]
fn launch() -> _ {
    dotenv().ok();
    let REDIRECT_URI = std::env::var("REDIRECT_URI").expect("yall forgot REDIRECT_URI");
    let CLIENT_ID = std::env::var("CLIENT_ID").expect("yall forgot CLIENT_ID");
    let CLIENT_SECRET = std::env::var("CLIENT_SECRET").expect("yall forgot CLIENT_SECRET");

    let client: Client = Client::new(REDIRECT_URI, CLIENT_ID, CLIENT_SECRET);
    let session_manager: SessionManager = SessionManager::new();
    let shared_session_manager: SharedSessionManager = SharedSessionManager::new(session_manager);
    rocket::build()
        .manage(client)
        .manage(shared_session_manager)
        .mount("/", routes![index, redirect_auth, return_sess, get_favicon])
}
